from http import HTTPStatus
from typing import Any, Optional, Union

import requests

from ...client import AuthenticatedClient, Client
from ...models.auth_token_verify_create_response_200 import (
    AuthTokenVerifyCreateResponse200,
)
from ...models.token_verify_request import TokenVerifyRequest
from ...types import UNSET, Response, Unset, check_nested_instance

QUERY_PARAMS = {}

REQUEST_BODY_TYPE = TokenVerifyRequest


def _get_kwargs(
    *,
    client: Union[AuthenticatedClient, Client],
    body: Union[
        TokenVerifyRequest,
        TokenVerifyRequest,
        TokenVerifyRequest,
    ],
) -> dict[str, Any]:
    headers: dict[str, Any] = client.get_headers()

    _kwargs: dict[str, Any] = {
        "url": f"{client.base_url}/auth/token/verify",
    }

    if check_nested_instance(body, TokenVerifyRequest):
        _json_body: dict[str, Any] = UNSET
        if not isinstance(body, Unset):
            _json_body = body.to_dict()

        _kwargs["json"] = _json_body
        headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Optional[AuthTokenVerifyCreateResponse200]:
    if response.status_code == 200:
        _response_200 = response.json()
        response_200: AuthTokenVerifyCreateResponse200
        if isinstance(_response_200, Unset):
            response_200 = UNSET
        else:
            response_200 = AuthTokenVerifyCreateResponse200.from_dict(_response_200)

        return response_200


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Response[AuthTokenVerifyCreateResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: Union[
        TokenVerifyRequest,
        TokenVerifyRequest,
        TokenVerifyRequest,
    ],
) -> Response[AuthTokenVerifyCreateResponse200]:
    """Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.

    Args:
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthTokenVerifyCreateResponse200]
    """

    kwargs = _get_kwargs(
        client=client,
        body=body,
    )

    response = requests.post(
        verify=client.verify_ssl,
        auth=client.auth,
        timeout=client.timeout,
        **kwargs,
    )
    response.raise_for_status()

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: Union[
        TokenVerifyRequest,
        TokenVerifyRequest,
        TokenVerifyRequest,
    ],
) -> Optional[AuthTokenVerifyCreateResponse200]:
    """Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.

    Args:
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthTokenVerifyCreateResponse200
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: Union[
        TokenVerifyRequest,
        TokenVerifyRequest,
        TokenVerifyRequest,
    ],
) -> Response[AuthTokenVerifyCreateResponse200]:
    """Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.

    Args:
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthTokenVerifyCreateResponse200]
    """

    kwargs = _get_kwargs(
        client=client,
        body=body,
    )

    async with client.get_async_session().post(
        verify_ssl=client.verify_ssl, raise_for_status=True, **kwargs
    ) as response:
        content = await response.read()
        resp = requests.Response()
        resp.status_code = response.status
        resp._content = content

    return _build_response(client=client, response=resp)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: Union[
        TokenVerifyRequest,
        TokenVerifyRequest,
        TokenVerifyRequest,
    ],
) -> Optional[AuthTokenVerifyCreateResponse200]:
    """Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.

    Args:
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):
        body (TokenVerifyRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthTokenVerifyCreateResponse200
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



import datetime
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="WorkflowsRetrieveResponse200")


@_attrs_define
class WorkflowsRetrieveResponse200(OSIDBModel):
    """
    Attributes:
        dt (Union[Unset, datetime.datetime]):
        env (Union[Unset, str]):
        revision (Union[Unset, str]):
        version (Union[Unset, str]):
    """

    dt: Union[Unset, datetime.datetime] = UNSET
    env: Union[Unset, str] = UNSET
    revision: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        dt: Union[Unset, str] = UNSET
        if not isinstance(self.dt, Unset):
            dt = self.dt.isoformat()

        env = self.env

        revision = self.revision

        version = self.version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(dt, Unset):
            field_dict["dt"] = dt
        if not isinstance(env, Unset):
            field_dict["env"] = env
        if not isinstance(revision, Unset):
            field_dict["revision"] = revision
        if not isinstance(version, Unset):
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _dt = d.pop("dt", UNSET)
        dt: Union[Unset, datetime.datetime]
        if isinstance(_dt, Unset):
            dt = UNSET
        else:
            dt = isoparse(_dt)

        env = d.pop("env", UNSET)

        revision = d.pop("revision", UNSET)

        version = d.pop("version", UNSET)

        workflows_retrieve_response_200 = cls(
            dt=dt,
            env=env,
            revision=revision,
            version=version,
        )

        workflows_retrieve_response_200.additional_properties = d
        return workflows_retrieve_response_200

    @staticmethod
    def get_fields():
        return {
            "dt": datetime.datetime,
            "env": str,
            "revision": str,
            "version": str,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import requests

from ...client import AuthenticatedClient, Client
from ...models.osidb_api_v1_audit_list_response_200 import (
    OsidbApiV1AuditListResponse200,
)
from ...types import UNSET, Response, Unset

QUERY_PARAMS = {
    "limit": int,
    "offset": int,
    "pgh_created_at": datetime.datetime,
    "pgh_label": str,
    "pgh_obj_model": str,
    "pgh_slug": str,
}


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    pgh_created_at: Union[Unset, datetime.datetime] = UNSET,
    pgh_label: Union[Unset, str] = UNSET,
    pgh_obj_model: Union[Unset, str] = UNSET,
    pgh_slug: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = client.get_headers()

    params: dict[str, Any] = {}

    params["limit"] = limit

    params["offset"] = offset

    json_pgh_created_at: Union[Unset, str] = UNSET
    if not isinstance(pgh_created_at, Unset):
        json_pgh_created_at = pgh_created_at.isoformat()

    params["pgh_created_at"] = json_pgh_created_at

    params["pgh_label"] = pgh_label

    params["pgh_obj_model"] = pgh_obj_model

    params["pgh_slug"] = pgh_slug

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "url": f"{client.base_url}/osidb/api/v1/audit",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Optional[OsidbApiV1AuditListResponse200]:
    if response.status_code == 200:
        _response_200 = response.json()
        response_200: OsidbApiV1AuditListResponse200
        if isinstance(_response_200, Unset):
            response_200 = UNSET
        else:
            response_200 = OsidbApiV1AuditListResponse200.from_dict(_response_200)

        return response_200


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Response[OsidbApiV1AuditListResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    pgh_created_at: Union[Unset, datetime.datetime] = UNSET,
    pgh_label: Union[Unset, str] = UNSET,
    pgh_obj_model: Union[Unset, str] = UNSET,
    pgh_slug: Union[Unset, str] = UNSET,
) -> Response[OsidbApiV1AuditListResponse200]:
    """basic view of audit history events

    Args:
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        pgh_created_at (Union[Unset, datetime.datetime]):
        pgh_label (Union[Unset, str]):
        pgh_obj_model (Union[Unset, str]):
        pgh_slug (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[OsidbApiV1AuditListResponse200]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        offset=offset,
        pgh_created_at=pgh_created_at,
        pgh_label=pgh_label,
        pgh_obj_model=pgh_obj_model,
        pgh_slug=pgh_slug,
    )

    response = requests.get(
        verify=client.verify_ssl,
        auth=client.auth,
        timeout=client.timeout,
        **kwargs,
    )
    response.raise_for_status()

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    pgh_created_at: Union[Unset, datetime.datetime] = UNSET,
    pgh_label: Union[Unset, str] = UNSET,
    pgh_obj_model: Union[Unset, str] = UNSET,
    pgh_slug: Union[Unset, str] = UNSET,
) -> Optional[OsidbApiV1AuditListResponse200]:
    """basic view of audit history events

    Args:
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        pgh_created_at (Union[Unset, datetime.datetime]):
        pgh_label (Union[Unset, str]):
        pgh_obj_model (Union[Unset, str]):
        pgh_slug (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        OsidbApiV1AuditListResponse200
    """

    return sync_detailed(
        client=client,
        limit=limit,
        offset=offset,
        pgh_created_at=pgh_created_at,
        pgh_label=pgh_label,
        pgh_obj_model=pgh_obj_model,
        pgh_slug=pgh_slug,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    pgh_created_at: Union[Unset, datetime.datetime] = UNSET,
    pgh_label: Union[Unset, str] = UNSET,
    pgh_obj_model: Union[Unset, str] = UNSET,
    pgh_slug: Union[Unset, str] = UNSET,
) -> Response[OsidbApiV1AuditListResponse200]:
    """basic view of audit history events

    Args:
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        pgh_created_at (Union[Unset, datetime.datetime]):
        pgh_label (Union[Unset, str]):
        pgh_obj_model (Union[Unset, str]):
        pgh_slug (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[OsidbApiV1AuditListResponse200]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        offset=offset,
        pgh_created_at=pgh_created_at,
        pgh_label=pgh_label,
        pgh_obj_model=pgh_obj_model,
        pgh_slug=pgh_slug,
    )

    async with client.get_async_session().get(
        verify_ssl=client.verify_ssl, raise_for_status=True, **kwargs
    ) as response:
        content = await response.read()
        resp = requests.Response()
        resp.status_code = response.status
        resp._content = content

    return _build_response(client=client, response=resp)


async def asyncio(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    pgh_created_at: Union[Unset, datetime.datetime] = UNSET,
    pgh_label: Union[Unset, str] = UNSET,
    pgh_obj_model: Union[Unset, str] = UNSET,
    pgh_slug: Union[Unset, str] = UNSET,
) -> Optional[OsidbApiV1AuditListResponse200]:
    """basic view of audit history events

    Args:
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        pgh_created_at (Union[Unset, datetime.datetime]):
        pgh_label (Union[Unset, str]):
        pgh_obj_model (Union[Unset, str]):
        pgh_slug (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        OsidbApiV1AuditListResponse200
    """

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            offset=offset,
            pgh_created_at=pgh_created_at,
            pgh_label=pgh_label,
            pgh_obj_model=pgh_obj_model,
            pgh_slug=pgh_slug,
        )
    ).parsed



from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.state_enum import StateEnum
from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="FlawCollaboratorRequest")


@_attrs_define
class FlawCollaboratorRequest(OSIDBModel):
    """FlawCollaborator serializer

    Attributes:
        flaw (UUID):
        label (str):
        state (Union[Unset, StateEnum]):
        contributor (Union[Unset, str]):
        relevant (Union[Unset, bool]):
    """

    flaw: UUID
    label: str
    state: Union[Unset, StateEnum] = UNSET
    contributor: Union[Unset, str] = UNSET
    relevant: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        flaw: str = UNSET
        if not isinstance(self.flaw, Unset):
            flaw = str(self.flaw)

        label = self.label

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = StateEnum(self.state).value

        contributor = self.contributor

        relevant = self.relevant

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(flaw, Unset):
            field_dict["flaw"] = flaw
        if not isinstance(label, Unset):
            field_dict["label"] = label
        if not isinstance(state, Unset):
            field_dict["state"] = state
        if not isinstance(contributor, Unset):
            field_dict["contributor"] = contributor
        if not isinstance(relevant, Unset):
            field_dict["relevant"] = relevant

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _flaw = d.pop("flaw", UNSET)
        flaw: UUID
        if isinstance(_flaw, Unset):
            flaw = UNSET
        else:
            flaw = _flaw if isinstance(_flaw, UUID) else UUID(_flaw)

        label = d.pop("label", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, StateEnum]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = StateEnum(_state)

        contributor = d.pop("contributor", UNSET)

        relevant = d.pop("relevant", UNSET)

        flaw_collaborator_request = cls(
            flaw=flaw,
            label=label,
            state=state,
            contributor=contributor,
            relevant=relevant,
        )

        flaw_collaborator_request.additional_properties = d
        return flaw_collaborator_request

    @staticmethod
    def get_fields():
        return {
            "flaw": UUID,
            "label": str,
            "state": StateEnum,
            "contributor": str,
            "relevant": bool,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.blank_enum import BlankEnum
from ..models.not_affected_justification_enum import NotAffectedJustificationEnum
from ..models.special_handling_enum import SpecialHandlingEnum
from ..models.tracker_type import TrackerType
from ..types import UNSET, OSIDBModel, Unset

if TYPE_CHECKING:
    from ..models.alert import Alert
    from ..models.erratum import Erratum


T = TypeVar("T", bound="OsidbApiV1TrackersCreateResponse201")


@_attrs_define
class OsidbApiV1TrackersCreateResponse201(OSIDBModel):
    """
    Attributes:
        errata (list['Erratum']):
        ps_update_stream (str):
        status (str):
        resolution (str):
        not_affected_justification (Union[BlankEnum, NotAffectedJustificationEnum]):
        type_ (TrackerType):
        uuid (UUID):
        special_handling (list[SpecialHandlingEnum]):
        resolved_dt (Union[None, datetime.datetime]):
        embargoed (bool): The embargoed boolean attribute is technically read-only as it just indirectly modifies the
            ACLs but is mandatory as it controls the access to the resource.
        alerts (list['Alert']):
        created_dt (datetime.datetime):
        updated_dt (datetime.datetime): The updated_dt timestamp attribute is mandatory on update as it is used to
            detect mit-air collisions.
        affects (Union[Unset, list[UUID]]):
        dt (Union[Unset, datetime.datetime]):
        env (Union[Unset, str]):
        revision (Union[Unset, str]):
        version (Union[Unset, str]):
    """

    errata: list["Erratum"]
    ps_update_stream: str
    status: str
    resolution: str
    not_affected_justification: Union[BlankEnum, NotAffectedJustificationEnum]
    type_: TrackerType
    uuid: UUID
    special_handling: list[SpecialHandlingEnum]
    resolved_dt: Union[None, datetime.datetime]
    embargoed: bool
    alerts: list["Alert"]
    created_dt: datetime.datetime
    updated_dt: datetime.datetime
    affects: Union[Unset, list[UUID]] = UNSET
    dt: Union[Unset, datetime.datetime] = UNSET
    env: Union[Unset, str] = UNSET
    revision: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        errata: list[dict[str, Any]] = UNSET
        if not isinstance(self.errata, Unset):
            errata = []
            for errata_item_data in self.errata:
                errata_item: dict[str, Any] = UNSET
                if not isinstance(errata_item_data, Unset):
                    errata_item = errata_item_data.to_dict()

                errata.append(errata_item)

        ps_update_stream = self.ps_update_stream

        status = self.status

        resolution = self.resolution

        not_affected_justification: str
        if isinstance(self.not_affected_justification, Unset):
            not_affected_justification = UNSET
        elif isinstance(self.not_affected_justification, NotAffectedJustificationEnum):
            not_affected_justification = UNSET
            if not isinstance(self.not_affected_justification, Unset):
                not_affected_justification = NotAffectedJustificationEnum(
                    self.not_affected_justification
                ).value

        else:
            not_affected_justification = UNSET
            if not isinstance(self.not_affected_justification, Unset):
                not_affected_justification = BlankEnum(
                    self.not_affected_justification
                ).value

        type_: str = UNSET
        if not isinstance(self.type_, Unset):
            type_ = TrackerType(self.type_).value

        uuid: str = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        special_handling: list[str] = UNSET
        if not isinstance(self.special_handling, Unset):
            special_handling = []
            for special_handling_item_data in self.special_handling:
                special_handling_item: str = UNSET
                if not isinstance(special_handling_item_data, Unset):
                    special_handling_item = SpecialHandlingEnum(
                        special_handling_item_data
                    ).value

                special_handling.append(special_handling_item)

        resolved_dt: Union[None, str]
        if isinstance(self.resolved_dt, Unset):
            resolved_dt = UNSET
        elif isinstance(self.resolved_dt, datetime.datetime):
            resolved_dt = UNSET
            if not isinstance(self.resolved_dt, Unset):
                resolved_dt = self.resolved_dt.isoformat()

        else:
            resolved_dt = self.resolved_dt

        embargoed = self.embargoed

        alerts: list[dict[str, Any]] = UNSET
        if not isinstance(self.alerts, Unset):
            alerts = []
            for alerts_item_data in self.alerts:
                alerts_item: dict[str, Any] = UNSET
                if not isinstance(alerts_item_data, Unset):
                    alerts_item = alerts_item_data.to_dict()

                alerts.append(alerts_item)

        created_dt: str = UNSET
        if not isinstance(self.created_dt, Unset):
            created_dt = self.created_dt.isoformat()

        updated_dt: str = UNSET
        if not isinstance(self.updated_dt, Unset):
            updated_dt = self.updated_dt.isoformat()

        affects: Union[Unset, list[str]] = UNSET
        if not isinstance(self.affects, Unset):
            affects = []
            for affects_item_data in self.affects:
                affects_item: str = UNSET
                if not isinstance(affects_item_data, Unset):
                    affects_item = str(affects_item_data)

                affects.append(affects_item)

        dt: Union[Unset, str] = UNSET
        if not isinstance(self.dt, Unset):
            dt = self.dt.isoformat()

        env = self.env

        revision = self.revision

        version = self.version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(errata, Unset):
            field_dict["errata"] = errata
        if not isinstance(ps_update_stream, Unset):
            field_dict["ps_update_stream"] = ps_update_stream
        if not isinstance(status, Unset):
            field_dict["status"] = status
        if not isinstance(resolution, Unset):
            field_dict["resolution"] = resolution
        if not isinstance(not_affected_justification, Unset):
            field_dict["not_affected_justification"] = not_affected_justification
        if not isinstance(type_, Unset):
            field_dict["type"] = type_
        if not isinstance(uuid, Unset):
            field_dict["uuid"] = uuid
        if not isinstance(special_handling, Unset):
            field_dict["special_handling"] = special_handling
        if not isinstance(resolved_dt, Unset):
            field_dict["resolved_dt"] = resolved_dt
        if not isinstance(embargoed, Unset):
            field_dict["embargoed"] = embargoed
        if not isinstance(alerts, Unset):
            field_dict["alerts"] = alerts
        if not isinstance(created_dt, Unset):
            field_dict["created_dt"] = created_dt
        if not isinstance(updated_dt, Unset):
            field_dict["updated_dt"] = updated_dt
        if not isinstance(affects, Unset):
            field_dict["affects"] = affects
        if not isinstance(dt, Unset):
            field_dict["dt"] = dt
        if not isinstance(env, Unset):
            field_dict["env"] = env
        if not isinstance(revision, Unset):
            field_dict["revision"] = revision
        if not isinstance(version, Unset):
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.alert import Alert
        from ..models.erratum import Erratum

        d = src_dict.copy()
        errata = []
        _errata = d.pop("errata", UNSET)
        for errata_item_data in _errata or []:
            _errata_item = errata_item_data
            errata_item: Erratum
            if isinstance(_errata_item, Unset):
                errata_item = UNSET
            else:
                errata_item = Erratum.from_dict(_errata_item)

            errata.append(errata_item)

        ps_update_stream = d.pop("ps_update_stream", UNSET)

        status = d.pop("status", UNSET)

        resolution = d.pop("resolution", UNSET)

        def _parse_not_affected_justification(
            data: object,
        ) -> Union[BlankEnum, NotAffectedJustificationEnum]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _not_affected_justification_type_0 = data
                not_affected_justification_type_0: NotAffectedJustificationEnum
                if isinstance(_not_affected_justification_type_0, Unset):
                    not_affected_justification_type_0 = UNSET
                else:
                    not_affected_justification_type_0 = NotAffectedJustificationEnum(
                        _not_affected_justification_type_0
                    )

                return not_affected_justification_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            _not_affected_justification_type_1 = data
            not_affected_justification_type_1: BlankEnum
            if isinstance(_not_affected_justification_type_1, Unset):
                not_affected_justification_type_1 = UNSET
            else:
                not_affected_justification_type_1 = BlankEnum(
                    _not_affected_justification_type_1
                )

            return not_affected_justification_type_1

        not_affected_justification = _parse_not_affected_justification(
            d.pop("not_affected_justification", UNSET)
        )

        _type_ = d.pop("type", UNSET)
        type_: TrackerType
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = TrackerType(_type_)

        _uuid = d.pop("uuid", UNSET)
        uuid: UUID
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = _uuid if isinstance(_uuid, UUID) else UUID(_uuid)

        special_handling = []
        _special_handling = d.pop("special_handling", UNSET)
        for special_handling_item_data in _special_handling or []:
            _special_handling_item = special_handling_item_data
            special_handling_item: SpecialHandlingEnum
            if isinstance(_special_handling_item, Unset):
                special_handling_item = UNSET
            else:
                special_handling_item = SpecialHandlingEnum(_special_handling_item)

            special_handling.append(special_handling_item)

        def _parse_resolved_dt(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _resolved_dt_type_0 = data
                resolved_dt_type_0: datetime.datetime
                if isinstance(_resolved_dt_type_0, Unset):
                    resolved_dt_type_0 = UNSET
                else:
                    resolved_dt_type_0 = isoparse(_resolved_dt_type_0)

                return resolved_dt_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        resolved_dt = _parse_resolved_dt(d.pop("resolved_dt", UNSET))

        embargoed = d.pop("embargoed", UNSET)

        alerts = []
        _alerts = d.pop("alerts", UNSET)
        for alerts_item_data in _alerts or []:
            _alerts_item = alerts_item_data
            alerts_item: Alert
            if isinstance(_alerts_item, Unset):
                alerts_item = UNSET
            else:
                alerts_item = Alert.from_dict(_alerts_item)

            alerts.append(alerts_item)

        _created_dt = d.pop("created_dt", UNSET)
        created_dt: datetime.datetime
        if isinstance(_created_dt, Unset):
            created_dt = UNSET
        else:
            created_dt = isoparse(_created_dt)

        _updated_dt = d.pop("updated_dt", UNSET)
        updated_dt: datetime.datetime
        if isinstance(_updated_dt, Unset):
            updated_dt = UNSET
        else:
            updated_dt = isoparse(_updated_dt)

        affects = []
        _affects = d.pop("affects", UNSET)
        for affects_item_data in _affects or []:
            _affects_item = affects_item_data
            affects_item: UUID
            if isinstance(_affects_item, Unset):
                affects_item = UNSET
            else:
                affects_item = (
                    _affects_item
                    if isinstance(_affects_item, UUID)
                    else UUID(_affects_item)
                )

            affects.append(affects_item)

        _dt = d.pop("dt", UNSET)
        dt: Union[Unset, datetime.datetime]
        if isinstance(_dt, Unset):
            dt = UNSET
        else:
            dt = isoparse(_dt)

        env = d.pop("env", UNSET)

        revision = d.pop("revision", UNSET)

        version = d.pop("version", UNSET)

        osidb_api_v1_trackers_create_response_201 = cls(
            errata=errata,
            ps_update_stream=ps_update_stream,
            status=status,
            resolution=resolution,
            not_affected_justification=not_affected_justification,
            type_=type_,
            uuid=uuid,
            special_handling=special_handling,
            resolved_dt=resolved_dt,
            embargoed=embargoed,
            alerts=alerts,
            created_dt=created_dt,
            updated_dt=updated_dt,
            affects=affects,
            dt=dt,
            env=env,
            revision=revision,
            version=version,
        )

        osidb_api_v1_trackers_create_response_201.additional_properties = d
        return osidb_api_v1_trackers_create_response_201

    @staticmethod
    def get_fields():
        return {
            "errata": list["Erratum"],
            "ps_update_stream": str,
            "status": str,
            "resolution": str,
            "not_affected_justification": Union[
                BlankEnum, NotAffectedJustificationEnum
            ],
            "type": TrackerType,
            "uuid": UUID,
            "special_handling": list[SpecialHandlingEnum],
            "resolved_dt": Union[None, datetime.datetime],
            "embargoed": bool,
            "alerts": list["Alert"],
            "created_dt": datetime.datetime,
            "updated_dt": datetime.datetime,
            "affects": list[UUID],
            "dt": datetime.datetime,
            "env": str,
            "revision": str,
            "version": str,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, OSIDBModel, Unset

if TYPE_CHECKING:
    from ..models.exploits_api_v1_report_date_retrieve_response_200_action_required_item import (
        ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem,
    )
    from ..models.exploits_api_v1_report_date_retrieve_response_200_no_action_item import (
        ExploitsApiV1ReportDateRetrieveResponse200NoActionItem,
    )
    from ..models.exploits_api_v1_report_date_retrieve_response_200_not_relevant_item import (
        ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem,
    )


T = TypeVar("T", bound="ExploitsApiV1ReportDateRetrieveResponse200")


@_attrs_define
class ExploitsApiV1ReportDateRetrieveResponse200(OSIDBModel):
    """
    Attributes:
        action_required (Union[Unset, list['ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem']]):
        cutoff_date (Union[Unset, str]):
        dt (Union[Unset, datetime.datetime]):
        env (Union[Unset, str]):
        evaluated_cves (Union[Unset, int]):
        no_action (Union[Unset, list['ExploitsApiV1ReportDateRetrieveResponse200NoActionItem']]):
        not_relevant (Union[Unset, list['ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem']]):
        revision (Union[Unset, str]):
        version (Union[Unset, str]):
    """

    action_required: Union[
        Unset, list["ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem"]
    ] = UNSET
    cutoff_date: Union[Unset, str] = UNSET
    dt: Union[Unset, datetime.datetime] = UNSET
    env: Union[Unset, str] = UNSET
    evaluated_cves: Union[Unset, int] = UNSET
    no_action: Union[
        Unset, list["ExploitsApiV1ReportDateRetrieveResponse200NoActionItem"]
    ] = UNSET
    not_relevant: Union[
        Unset, list["ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem"]
    ] = UNSET
    revision: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action_required: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.action_required, Unset):
            action_required = []
            for action_required_item_data in self.action_required:
                action_required_item: dict[str, Any] = UNSET
                if not isinstance(action_required_item_data, Unset):
                    action_required_item = action_required_item_data.to_dict()

                action_required.append(action_required_item)

        cutoff_date = self.cutoff_date

        dt: Union[Unset, str] = UNSET
        if not isinstance(self.dt, Unset):
            dt = self.dt.isoformat()

        env = self.env

        evaluated_cves = self.evaluated_cves

        no_action: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.no_action, Unset):
            no_action = []
            for no_action_item_data in self.no_action:
                no_action_item: dict[str, Any] = UNSET
                if not isinstance(no_action_item_data, Unset):
                    no_action_item = no_action_item_data.to_dict()

                no_action.append(no_action_item)

        not_relevant: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.not_relevant, Unset):
            not_relevant = []
            for not_relevant_item_data in self.not_relevant:
                not_relevant_item: dict[str, Any] = UNSET
                if not isinstance(not_relevant_item_data, Unset):
                    not_relevant_item = not_relevant_item_data.to_dict()

                not_relevant.append(not_relevant_item)

        revision = self.revision

        version = self.version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(action_required, Unset):
            field_dict["action_required"] = action_required
        if not isinstance(cutoff_date, Unset):
            field_dict["cutoff_date"] = cutoff_date
        if not isinstance(dt, Unset):
            field_dict["dt"] = dt
        if not isinstance(env, Unset):
            field_dict["env"] = env
        if not isinstance(evaluated_cves, Unset):
            field_dict["evaluated_cves"] = evaluated_cves
        if not isinstance(no_action, Unset):
            field_dict["no_action"] = no_action
        if not isinstance(not_relevant, Unset):
            field_dict["not_relevant"] = not_relevant
        if not isinstance(revision, Unset):
            field_dict["revision"] = revision
        if not isinstance(version, Unset):
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.exploits_api_v1_report_date_retrieve_response_200_action_required_item import (
            ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem,
        )
        from ..models.exploits_api_v1_report_date_retrieve_response_200_no_action_item import (
            ExploitsApiV1ReportDateRetrieveResponse200NoActionItem,
        )
        from ..models.exploits_api_v1_report_date_retrieve_response_200_not_relevant_item import (
            ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem,
        )

        d = src_dict.copy()
        action_required = []
        _action_required = d.pop("action_required", UNSET)
        for action_required_item_data in _action_required or []:
            _action_required_item = action_required_item_data
            action_required_item: (
                ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem
            )
            if isinstance(_action_required_item, Unset):
                action_required_item = UNSET
            else:
                action_required_item = ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem.from_dict(
                    _action_required_item
                )

            action_required.append(action_required_item)

        cutoff_date = d.pop("cutoff_date", UNSET)

        _dt = d.pop("dt", UNSET)
        dt: Union[Unset, datetime.datetime]
        if isinstance(_dt, Unset):
            dt = UNSET
        else:
            dt = isoparse(_dt)

        env = d.pop("env", UNSET)

        evaluated_cves = d.pop("evaluated_cves", UNSET)

        no_action = []
        _no_action = d.pop("no_action", UNSET)
        for no_action_item_data in _no_action or []:
            _no_action_item = no_action_item_data
            no_action_item: ExploitsApiV1ReportDateRetrieveResponse200NoActionItem
            if isinstance(_no_action_item, Unset):
                no_action_item = UNSET
            else:
                no_action_item = (
                    ExploitsApiV1ReportDateRetrieveResponse200NoActionItem.from_dict(
                        _no_action_item
                    )
                )

            no_action.append(no_action_item)

        not_relevant = []
        _not_relevant = d.pop("not_relevant", UNSET)
        for not_relevant_item_data in _not_relevant or []:
            _not_relevant_item = not_relevant_item_data
            not_relevant_item: ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem
            if isinstance(_not_relevant_item, Unset):
                not_relevant_item = UNSET
            else:
                not_relevant_item = (
                    ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem.from_dict(
                        _not_relevant_item
                    )
                )

            not_relevant.append(not_relevant_item)

        revision = d.pop("revision", UNSET)

        version = d.pop("version", UNSET)

        exploits_api_v1_report_date_retrieve_response_200 = cls(
            action_required=action_required,
            cutoff_date=cutoff_date,
            dt=dt,
            env=env,
            evaluated_cves=evaluated_cves,
            no_action=no_action,
            not_relevant=not_relevant,
            revision=revision,
            version=version,
        )

        exploits_api_v1_report_date_retrieve_response_200.additional_properties = d
        return exploits_api_v1_report_date_retrieve_response_200

    @staticmethod
    def get_fields():
        return {
            "action_required": list[
                "ExploitsApiV1ReportDateRetrieveResponse200ActionRequiredItem"
            ],
            "cutoff_date": str,
            "dt": datetime.datetime,
            "env": str,
            "evaluated_cves": int,
            "no_action": list["ExploitsApiV1ReportDateRetrieveResponse200NoActionItem"],
            "not_relevant": list[
                "ExploitsApiV1ReportDateRetrieveResponse200NotRelevantItem"
            ],
            "revision": str,
            "version": str,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import requests

from ...client import AuthenticatedClient, Client
from ...models.flaw_reference_post_request import FlawReferencePostRequest
from ...models.osidb_api_v1_flaws_references_create_response_201 import (
    OsidbApiV1FlawsReferencesCreateResponse201,
)
from ...types import UNSET, Response, Unset, check_nested_instance

QUERY_PARAMS = {}

REQUEST_BODY_TYPE = FlawReferencePostRequest


def _get_kwargs(
    flaw_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        FlawReferencePostRequest,
        FlawReferencePostRequest,
        FlawReferencePostRequest,
    ],
) -> dict[str, Any]:
    headers: dict[str, Any] = client.get_headers()

    _kwargs: dict[str, Any] = {
        "url": f"{client.base_url}/osidb/api/v1/flaws/{flaw_id}/references".format(
            flaw_id=flaw_id,
        ),
    }

    if check_nested_instance(body, FlawReferencePostRequest):
        _json_body: dict[str, Any] = UNSET
        if not isinstance(body, Unset):
            _json_body = body.to_dict()

        _kwargs["json"] = _json_body
        headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Optional[OsidbApiV1FlawsReferencesCreateResponse201]:
    if response.status_code == 201:
        _response_201 = response.json()
        response_201: OsidbApiV1FlawsReferencesCreateResponse201
        if isinstance(_response_201, Unset):
            response_201 = UNSET
        else:
            response_201 = OsidbApiV1FlawsReferencesCreateResponse201.from_dict(
                _response_201
            )

        return response_201


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: requests.Response
) -> Response[OsidbApiV1FlawsReferencesCreateResponse201]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    flaw_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        FlawReferencePostRequest,
        FlawReferencePostRequest,
        FlawReferencePostRequest,
    ],
) -> Response[OsidbApiV1FlawsReferencesCreateResponse201]:
    """
    Args:
        flaw_id (UUID):
        bugzilla_api_key (str):
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[OsidbApiV1FlawsReferencesCreateResponse201]
    """

    kwargs = _get_kwargs(
        flaw_id=flaw_id,
        client=client,
        body=body,
    )

    response = requests.post(
        verify=client.verify_ssl,
        auth=client.auth,
        timeout=client.timeout,
        **kwargs,
    )
    response.raise_for_status()

    return _build_response(client=client, response=response)


def sync(
    flaw_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        FlawReferencePostRequest,
        FlawReferencePostRequest,
        FlawReferencePostRequest,
    ],
) -> Optional[OsidbApiV1FlawsReferencesCreateResponse201]:
    """
    Args:
        flaw_id (UUID):
        bugzilla_api_key (str):
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        OsidbApiV1FlawsReferencesCreateResponse201
    """

    return sync_detailed(
        flaw_id=flaw_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    flaw_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        FlawReferencePostRequest,
        FlawReferencePostRequest,
        FlawReferencePostRequest,
    ],
) -> Response[OsidbApiV1FlawsReferencesCreateResponse201]:
    """
    Args:
        flaw_id (UUID):
        bugzilla_api_key (str):
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[OsidbApiV1FlawsReferencesCreateResponse201]
    """

    kwargs = _get_kwargs(
        flaw_id=flaw_id,
        client=client,
        body=body,
    )

    async with client.get_async_session().post(
        verify_ssl=client.verify_ssl, raise_for_status=True, **kwargs
    ) as response:
        content = await response.read()
        resp = requests.Response()
        resp.status_code = response.status
        resp._content = content

    return _build_response(client=client, response=resp)


async def asyncio(
    flaw_id: UUID,
    *,
    client: AuthenticatedClient,
    body: Union[
        FlawReferencePostRequest,
        FlawReferencePostRequest,
        FlawReferencePostRequest,
    ],
) -> Optional[OsidbApiV1FlawsReferencesCreateResponse201]:
    """
    Args:
        flaw_id (UUID):
        bugzilla_api_key (str):
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer
        body (FlawReferencePostRequest): FlawReference serializer

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        OsidbApiV1FlawsReferencesCreateResponse201
    """

    return (
        await asyncio_detailed(
            flaw_id=flaw_id,
            client=client,
            body=body,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, OSIDBModel, Unset

if TYPE_CHECKING:
    from ..models.affect import Affect
    from ..models.ps_stream_selection import PsStreamSelection


T = TypeVar("T", bound="ModuleComponent")


@_attrs_define
class ModuleComponent(OSIDBModel):
    """
    Attributes:
        ps_module (str):
        ps_component (str):
        streams (list['PsStreamSelection']):
        selected (bool):
        affect (Affect): Affect serializer
    """

    ps_module: str
    ps_component: str
    streams: list["PsStreamSelection"]
    selected: bool
    affect: "Affect"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ps_module = self.ps_module

        ps_component = self.ps_component

        streams: list[dict[str, Any]] = UNSET
        if not isinstance(self.streams, Unset):
            streams = []
            for streams_item_data in self.streams:
                streams_item: dict[str, Any] = UNSET
                if not isinstance(streams_item_data, Unset):
                    streams_item = streams_item_data.to_dict()

                streams.append(streams_item)

        selected = self.selected

        affect: dict[str, Any] = UNSET
        if not isinstance(self.affect, Unset):
            affect = self.affect.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(ps_module, Unset):
            field_dict["ps_module"] = ps_module
        if not isinstance(ps_component, Unset):
            field_dict["ps_component"] = ps_component
        if not isinstance(streams, Unset):
            field_dict["streams"] = streams
        if not isinstance(selected, Unset):
            field_dict["selected"] = selected
        if not isinstance(affect, Unset):
            field_dict["affect"] = affect

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.affect import Affect
        from ..models.ps_stream_selection import PsStreamSelection

        d = src_dict.copy()
        ps_module = d.pop("ps_module", UNSET)

        ps_component = d.pop("ps_component", UNSET)

        streams = []
        _streams = d.pop("streams", UNSET)
        for streams_item_data in _streams or []:
            _streams_item = streams_item_data
            streams_item: PsStreamSelection
            if isinstance(_streams_item, Unset):
                streams_item = UNSET
            else:
                streams_item = PsStreamSelection.from_dict(_streams_item)

            streams.append(streams_item)

        selected = d.pop("selected", UNSET)

        _affect = d.pop("affect", UNSET)
        affect: Affect
        if isinstance(_affect, Unset):
            affect = UNSET
        else:
            affect = Affect.from_dict(_affect)

        module_component = cls(
            ps_module=ps_module,
            ps_component=ps_component,
            streams=streams,
            selected=selected,
            affect=affect,
        )

        module_component.additional_properties = d
        return module_component

    @staticmethod
    def get_fields():
        return {
            "ps_module": str,
            "ps_component": str,
            "streams": list["PsStreamSelection"],
            "selected": bool,
            "affect": Affect,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.exploit_only_report_data_source_enum import (
    ExploitOnlyReportDataSourceEnum,
)
from ..models.maturity_preliminary_enum import MaturityPreliminaryEnum
from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="ExploitOnlyReportData")


@_attrs_define
class ExploitOnlyReportData(OSIDBModel):
    """
    Attributes:
        cve (str):
        source (ExploitOnlyReportDataSourceEnum):
        maturity_preliminary (MaturityPreliminaryEnum):
        flaw (bool):
        date (Union[None, Unset, datetime.date]):
        reference (Union[Unset, str]):  Default: 'N/A'.
    """

    cve: str
    source: ExploitOnlyReportDataSourceEnum
    maturity_preliminary: MaturityPreliminaryEnum
    flaw: bool
    date: Union[None, Unset, datetime.date] = UNSET
    reference: Union[Unset, str] = "N/A"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cve = self.cve

        source: str = UNSET
        if not isinstance(self.source, Unset):
            source = ExploitOnlyReportDataSourceEnum(self.source).value

        maturity_preliminary: int = UNSET
        if not isinstance(self.maturity_preliminary, Unset):
            maturity_preliminary = MaturityPreliminaryEnum(
                self.maturity_preliminary
            ).value

        flaw = self.flaw

        date: Union[None, Unset, str]
        if isinstance(self.date, Unset):
            date = UNSET
        elif isinstance(self.date, datetime.date):
            date = UNSET
            if not isinstance(self.date, Unset):
                date = self.date.isoformat()

        else:
            date = self.date

        reference = self.reference

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(cve, Unset):
            field_dict["cve"] = cve
        if not isinstance(source, Unset):
            field_dict["source"] = source
        if not isinstance(maturity_preliminary, Unset):
            field_dict["maturity_preliminary"] = maturity_preliminary
        if not isinstance(flaw, Unset):
            field_dict["flaw"] = flaw
        if not isinstance(date, Unset):
            field_dict["date"] = date
        if not isinstance(reference, Unset):
            field_dict["reference"] = reference

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        cve = d.pop("cve", UNSET)

        _source = d.pop("source", UNSET)
        source: ExploitOnlyReportDataSourceEnum
        if isinstance(_source, Unset):
            source = UNSET
        else:
            source = ExploitOnlyReportDataSourceEnum(_source)

        _maturity_preliminary = d.pop("maturity_preliminary", UNSET)
        maturity_preliminary: MaturityPreliminaryEnum
        if isinstance(_maturity_preliminary, Unset):
            maturity_preliminary = UNSET
        else:
            maturity_preliminary = MaturityPreliminaryEnum(_maturity_preliminary)

        flaw = d.pop("flaw", UNSET)

        def _parse_date(data: object) -> Union[None, Unset, datetime.date]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _date_type_0 = data
                date_type_0: datetime.date
                if isinstance(_date_type_0, Unset):
                    date_type_0 = UNSET
                else:
                    date_type_0 = isoparse(_date_type_0).date()

                return date_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.date], data)

        date = _parse_date(d.pop("date", UNSET))

        reference = d.pop("reference", UNSET)

        exploit_only_report_data = cls(
            cve=cve,
            source=source,
            maturity_preliminary=maturity_preliminary,
            flaw=flaw,
            date=date,
            reference=reference,
        )

        exploit_only_report_data.additional_properties = d
        return exploit_only_report_data

    @staticmethod
    def get_fields():
        return {
            "cve": str,
            "source": ExploitOnlyReportDataSourceEnum,
            "maturity_preliminary": MaturityPreliminaryEnum,
            "flaw": bool,
            "date": Union[None, datetime.date],
            "reference": str,
        }

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OsidbApiV1FlawsListRequiresCveDescription(str, Enum):
    APPROVED = "APPROVED"
    REJECTED = "REJECTED"
    REQUESTED = "REQUESTED"
    VALUE_0 = ""

    def __str__(self) -> str:
        return str(self.value)
